
%{

#include <stdio.h>
#include "tree.h"

#define YYDEBUG 0

  extern FILE *yyin;
  extern int linenum;
  extern int colnum;

  yyerror(char *s) {
    fflush(stdout);
    fprintf(stderr,"%s in line %d and in column %d\n",s,linenum,colnum);
    fflush(stderr);
    exit(-1);
  }  

void fppNode(FILE *f, Node *n)
{
  if (n->type==LEAF) fprintf(f,"%s",n->label);
  else {
    fprintf(f,"(");
    fppNode(f,n->l);
    fprintf(f," ");
    fppNode(f,n->r);
    fprintf(f,")");    
  }
}

void ppTree(Tree *t)
{
  printf("Tree %s: ",t->name);
  fppNode(stdout,t->root);
}

%}

%union { 
  Node *n;
  Tree *t;
  char *s;
}

%token LPAR RPAR TREE
%token <s> LABEL 

%type <n> Node
%type <t> Tree

%%

input: Trees
;

Trees: 
       Trees Tree
       {
	 printf("Read");
	 ppTree($2);
	 genLaTeX($2);
	 printf("\n");
       }
       | {} 
;

Tree:
       LABEL Node
         {  
 	       $$=(Tree*)malloc(sizeof(Tree));
	       $$->name=$1; 
	       $$->root = $2;
	 }       
;


Node: 
        LABEL         
        {
	  $$=(Node*)malloc(sizeof(Node));
	  $$->type = LEAF;
	  $$->label = $1;
	}
	| 
	LPAR Node Node RPAR 
        { 
	  $$=(Node*)malloc(sizeof(Node));
	  $$->type=INT;
	  $$->l=$2;
	  $$->r=$3;
	}	     
;

%%

	
FILE *texout;

void genLaTeXNode(Node *n)
{
  switch (n->type)
    {
    case LEAF: 
      fprintf(texout,"{\\TR{%s}}",n->label);
      break;
    case INT: 
      fprintf(texout,"\\pstree{\\TR{"); 
      fppNode(texout,n);
      fprintf(texout,"}}\n{");
      genLaTeXNode(n->l);
      genLaTeXNode(n->r);
      fprintf(texout,"}\n");
      break;
    }
}

void genLaTeX(Tree *t)
{
  fprintf(texout,"\\begin{figure}[!htbp]\n");
  fprintf(texout,"\\centering\n");
  genLaTeXNode(t->root);
  fprintf(texout,"\\caption{%s}\n",t->name);
  fprintf(texout,"\\end{figure}\n");
}

int main(int argc, char **argv)
{

  texout = fopen("trees.tex","w");  

  fprintf(texout,"\\documentclass{article}\n");

  fprintf(texout,"\\usepackage[latin1]{inputenc}\n");
  fprintf(texout,"\\usepackage{calc}     \n");
  fprintf(texout,"\\usepackage{graphicx} \n");
  fprintf(texout,"\\usepackage{ifthen}   \n");
  fprintf(texout,"\\usepackage{subfigure}\n");
  fprintf(texout,"\\usepackage{pst-all}  \n");
  fprintf(texout,"\\usepackage{pst-poly} \n");
  fprintf(texout,"\\usepackage{multido}  \n");
  fprintf(texout,"\\begin{document}    \n");

  yyparse();
  
  fprintf(texout,"\\end{document}    \n");
  
  fclose(texout);
  return 0;
}





